// An extension of DirectManipDot in which we can control a
// whole army of dots.  The main difference is to maintain
// an array of objects containing positions and colours
// as our model.  This has the side effect that a boolean isn't
// sufficient to track whether we're dragging.  Instead, we use
// an integer (i.e., an array index) to keep track of *what*
// we're dragging.

int num_dots = 15;

class Dot
{
  int x;
  int y;
  color col;
  
  Dot( int x, int y, color col )
  {
    this.x = x;
    this.y = y;
    this.col = col;
  }
}

Dot[] dots;
int which_drag;
float radius = 20.0;

void setup()
{
  size( 500, 500 );
  
  // Generate a bunch of random dots.
  dots = new Dot[ num_dots ];
  for ( int idx = 0; idx < num_dots; ++idx ) {
    dots[idx] = new Dot( 
      int( random( width ) ),
      int( random( height ) ),
      color( random( 255 ), random( 255 ), random( 255 ) ) ); 
  }
  
  // Right now, we're not dragging anything.  Use -1 
  // to keep track of that.  -1 is never a valid array
  // index, so we won't get confused (but always check
  // if it's -1 before using to access any arrays!).
  which_drag = -1;
}

void draw()
{
  background( 220 );
  for ( int idx = 0; idx < num_dots; ++idx ) {
    if ( which_drag == idx ) {
      stroke( 255, 0, 0 );
    } else {
      stroke( 0, 0, 0 );
    }
    Dot d = dots[idx];
    fill( d.col );
    ellipse( d.x, d.y, 2.0*radius, 2.0*radius );
  }
}

void mousePressed()
{
  // Need to do a hit test against all the dots.
  
  // Puzzle: we'd get better behaviour if we ran this
  // loop in *reverse* over the dots.  Why??
  for ( int idx = 0; idx < num_dots; ++idx ) {
    Dot d = dots[idx];
    if( dist( d.x, d.y, mouseX, mouseY ) <= radius ) {
      which_drag = idx;
      break;
    }
  }
}

void mouseDragged()
{
  if ( which_drag != -1 ) {
    Dot d = dots[which_drag];
    d.x += mouseX - pmouseX;
    d.y += mouseY - pmouseY;
  }
}

void mouseReleased()
{
  which_drag = -1;
}